function [unit_data] = AggregateMOCgenericMK2(t,H,B,R,unit_data,k,dt,qinitial)

%% Fluid properties
g = 9.81;                                   %gravity acceleration constant
%% TURBINE PROPERTIES
Qr =  unit_data.Qr;                         %Rated flow at design point per turbine
nref = 600;
omegar = nref*2*pi/60;
%% CREATION OF BASE EULER TURBINE
U2r = 40;
turbacc = 1.1;
%reac = 0.5;
%eta = 0.96;
Hred = sqrt(2*g*unit_data.Hr);
% C1u = sqrt(eta - reac)*Hred;
% U1r = eta/(2*C1u/Hred)*Hred;
% r1 = U1r/(omegar);
% r2 = (U2r)/omegar;
% sigmageo = (2*r1)^2/8*(1-(r2/r1)^2)*2*(omegar/Hred)^2;
% r2 = sqrt((2*r1)^2-4*(sigmageo)/(2*(omegar/Hred)^2))/2;
% C2m = Qr/(pi*(r2^2));
% C1m = C2m/turbacc;
% b = Qr/(C1m*2*pi*r1);
% alpha1r = atan(C1m/C1u);
% beta2 = atan(C2m/U2r);
% V2 = (C2m/Hred)/sin(beta2)*Hred;
% C1r = (C1m/Hred)/sin(alpha1r)*Hred;
% beta1r = atan(C1m/(C1r-C1u));
% sigmaeuler = (1 - (2*(U2r/Hred)^2))/(1 + (2*(U2r/Hred)^2));
% DATA BELOW SET FROM EXTERNAL EXCEL SHEET. EQUIVALENT TO COMPUTATIONS
% ABOVE
C1u = 0.678233*Hred;
U1r = 0.7077*Hred;
r1 = 2.31877/2;
r2 = 1.2732/2;
sigmageo = 0.3498;
C2m = Qr/(pi*(r2^2));
C1m = C2m/turbacc;
b = Qr/(C1m*2*pi*r1);
alpha1r =0.0615;
beta2 = 0.1177;
C1r = 0.6795*Hred;
beta1r = 0.9562;

%% STEADY STATE INITIATION OF VARIABLES
%Loss variables
Rm = unit_data.Rm;
Ra = unit_data.Ra;
Rf = unit_data.Rf;
%Generator variables
omegagrid = 2*pi*50;                %Reference grid omega
P = 2*(omegagrid/omegar);           %Generator poles
Pratedgen = unit_data.Pref*10^6;    %Rated power generator (USER SET)
Tratedgen = Pratedgen/omegagrid;    %Generator magnetic torque (Geared by P/2)
qdimcr = 1*((1 + cot(alpha1r)*tan(beta1r))/(1 + cot(alpha1r)*tan(beta1r)));
hydraulicloss0 = (1-((Rf*qinitial^2 + Ra*(qinitial-qdimcr)^2)/(H(end)/unit_data.Hr)));
Prated = Pratedgen/((hydraulicloss0 - Rm)*qinitial);   %Rated power turbine
Trated = Prated/omegar;             %Rated torque turbine
Urated = unit_data.Eref;            %Rated voltage generator (USER SET)
korated = Urated/((P/2)*omegar);    %Rated magnetic constant
Irated = Tratedgen/korated;         %Rated voltage generator

PgenMAX = 90*10^6;                  %Generator maximum output
TgenMAX = PgenMAX/omegagrid;        %Generator maximum torque
deltaMAX = pi/4;                    %Displacement angle at maximum output
deltaref = asin((Tratedgen/TgenMAX)*sin(deltaMAX));

Rgridref = Urated/Irated;           
Rgrid = Rgridref;
%% REGULATION CONSTANTS
Uregref = Urated;                   %Set reference voltage for governor
Iregref = Irated;                   %Set reference current for governor
kapparegref = 1;                    %Set reference kappa for governor
%% EQUATION CONSTANTS AND SIMPLIFICATIONS
psi = U2r^2/(g*unit_data.Hr);       %Turbine pressure number
ksi2 = (psi + 1)/(cos(alpha1r));    %Turbine dimentionless machine constant
%Jpolar = mrotor*(0.735*r1);         %Rotor polar moment of inertia
Jpolar = unit_data.J;
Ta = Jpolar*(omegar^2/unit_data.Pnom);      %Rotor time constant

sigmaloss = -(1 - psi*(1 - Rf) - 2*Rm)/(2*Rf*((1+Rm)/(1-Rf)) - psi*(1 - Rf) - 1-2*Rm);

%Statics
deltat = unit_data.deltat;                  %Transient speed droop turbine (Kp = 1/deltat). Lower is faster
Td = unit_data.Ti;                          %Integration time
deltab = unit_data.deltab;                  %Permanent speed droop turbine
Tk = 0.000001;                              %Servo time constant (set to instant reaction)
deltatg = unit_data.deltatg;                %Transient speed droop generator
Tdg = unit_data.genTi;                      %Integration time generator
deltabg = unit_data.deltabg;                %Permanent speed droop generator
md = 0.02;                                  %Magnetic damping constant

K1 = 1/(deltat*nref);                       %turbine constant1
K2 = 1/(deltat*Td);                         %turbine constant2
K3 = (deltab*Tk + deltat*Td)/(deltat*Td);   %turbine constant3
K4 = deltab/(deltat*Td);                    %turbine constant4
K5 = 1/(deltatg*Uregref);                   %generator constant1
K6 = 1/(deltatg*Tdg*Uregref);               %generator constant2
K7 = 1/(deltabg*Iregref);                   %generator constant3
%% EXTRACTING VALUES FOR PREVIOUS TIMESTEP AND SETTING STEADY-STATE VALUES
if t == 0                               %If condition is initiation
    Uprev = Urated;
    omegadimprev = 1;
    kappaprev = qinitial;
    koprev = korated;
    cprev = 0;
    deltaprev = deltaref;
    unit_data.flow(1) = qinitial*Qr;
    qdimprev = qinitial;
    Iprev = Irated;
    Tgprev = Tratedgen*(P/2);
    
else
    Uprev = unit_data.gen_voltage(k-1);
	omegadimprev = unit_data.turb_omega(k-1)/omegar;
    kappaprev = unit_data.kappa(k-1);
    koprev = unit_data.gen_ko(k-1);
    cprev = unit_data.c(k-1);
    deltaprev = unit_data.gen_delta(k-1);
    qdimprev = unit_data.flow(k-1)/Qr;
    Iprev = unit_data.gen_current(k-1);
    Tgprev = ((sin(deltaprev)/sin(deltaref))*Tratedgen*(P/2));
end
%% SETTING SIMULATION SCHEDULE
eventtime1 = unit_data.simdata.t1;
eventtime2 = unit_data.simdata.t2;
eventtype1 = unit_data.simdata.mode1;
eventtype2 = unit_data.simdata.mode2;
residualevent = unit_data.simdata.mode3;

if unit_data.unitID == 1
    gridfactor1 = 1;
    gridfactor2 = 1;
    residualgridfactor = 1;
elseif unit_data.unitID == 2
    gridfactor1 = 1;
    gridfactor2 = 1;
    residualgridfactor = 1;
end

%CHANGES IN CONDITIONS MUST BE PUT IN BLOCKS BELOW
omegaregref = omegar;
if t <= eventtime1
    scheme = eventtype1;
    Rgrid = gridfactor1*Rgridref;
elseif t > eventtime1 && t <= eventtime2
    scheme = eventtype2;
    Rgrid = gridfactor2*Rgridref;
    omegagrid = 2*pi*50.2;
    %omegagrid = 2*pi*(50 + 0.3*sin(2*pi*((t-eventtime1 - dt)/1.2)))
    %omegaregref = omegaprev;
%     if t >= eventtime1 + 1
%         qinitial = 1.1
%     end
    global omegagridprev;
    global omegagridholder
%     if omegagrid < omegagridprev
%         omegagrid = omegagridprev;
%     end
    omegagridprev = omegagrid;
    omegagridholder(k) = omegagrid;             %Saving frequency history
elseif t > eventtime2
    scheme = residualevent;
    Rgrid = residualgridfactor*Rgridref;
    omegagrid = 2*pi*49.7;
    %omegagrid = 2*pi*(50 + 0.1*sin(100*(eventtime2-eventtime1 - dt)));
end
%% DECLARATION OF SYMBOLIC VARIABLES FOR SOLVER
syms kappa
syms qdim
syms omegadim
syms ko
syms I
syms U
syms c
syms delta
syms Tg
%% TIME-STEP SPECIFIC VALUES
if t < 0.05
	Cp = H(end-1) + Qr*qinitial*(B - R*abs(Qr*qinitial));
else
	Cp = H(end-1) + unit_data.flow(k-1)*(B - R*abs(unit_data.flow(k-1)));
end
if ~isfield(unit_data,'Hs')
    unit_data.Hs = Cp - B*Qr*qinitial - unit_data.Hr;
end
mr = ksi2*1*(cos(alpha1r) + tan(alpha1r)*sin(alpha1r));
alpha1 = (asin(kappa*sin(alpha1r)));
ms = ((1+psi)/mr)*ksi2*(abs(qdim)/kappa)*(cos(alpha1) + tan(alpha1r)*sin(alpha1));
%qdimc = ((Ain)*r1*tan(beta1r))/(1 + cot(alpha1)*tan(beta1r));
qdimc = omegadim*((1 + cot(alpha1r)*tan(beta1r))/(1 + cot(alpha1)*tan(beta1r)));

%% ------------ ONLINE DYNAMIC REGULATION ----------------
if strcmp(scheme,'Regulation')

    % DIFFERENTIAL EQUATIONS
    %Waterway equation
    EQ1 = Cp - B*Qr*qdim - unit_data.Hs - unit_data.Hr*(1/(1+sigmaloss))*((qdim/kappa)^2 + sigmaloss*(omegadim^2));
    %Turbine torque equation
    EQ2 = qdim*(ms - psi*omegadim)*(1-((Rf*qdim^2 + Ra*abs((qdim-qdimc))^2)/(H(end)/unit_data.Hr))) - Tg/Trated  - Rm*omegadim^2 - md*((delta-deltaprev)/dt) - (Ta/dt)*(omegadim - omegadimprev);
    %Generator angle equation in island mode
    %EQ3 = dt*((P/2)*omegadim*omegar - U/ko) - (delta - deltaprev);
    %Generator angle equation in static mode
    EQ3 = dt*((P/2)*omegadim*omegar - omegagrid) - (delta - deltaprev);
    % Kappa regulator equation
    EQ4 = dt*(qinitial/Tk)*(-K1*((omegadim*omegar - omegadimprev*omegar)/dt)*(30/pi) + K2*((omegaregref - omegadim*omegar)/omegaregref) - K3*c - K4*(qinitial - kappa)) - (c - cprev);
    %Voltage regulator equation
    EQ5 = -dt*(-K5*((U - Uprev)/dt) + K6*(Uregref + K7*(I - Iregref) - U)) - (ko - koprev);
    %Integration of kappa
    EQ6 = kappaprev + dt*c - kappa;
    % ALGEBRAIC EQUATIONS
    %Grid ohms law
    EQ7 = Rgrid*((2*Tg)/(ko*P)) - U;
    %Generator torque vs output equation
    EQ9 = ((sin(delta)/sin(deltaMAX))*TgenMAX*(P/2)) - Tg;
    %Generator torque equation
    EQ8 = ko*I*(P/2) - Tg;
    
    %Solving equation
    nsolution = newton_n_dim(0.1,[qdimprev,kappaprev,omegadimprev,cprev,deltaprev,Uprev,koprev,Tgprev,Iprev],[qdim,kappa,omegadim,c,delta,U,ko,Tg,I],[EQ1;EQ2;EQ3;EQ4;EQ5;EQ6;EQ7;EQ8;EQ9]);
    
    solution(1) = double(nsolution(1));
    solution(2) = double(nsolution(2));
    solution(3) = double(nsolution(3));
    solution(4) = double(nsolution(4));
    solution(5) = double(nsolution(5));
    solution(6) = double(nsolution(6));
    solution(7) = double(nsolution(7));
    solution(8) = double(nsolution(8));
    solution(9) = double(nsolution(9));
    
    unit_data.flow(k) = solution(1)*Qr;
    unit_data.turb_omega(k) = solution(3)*omegar;
    unit_data.c(k) = solution(4);
    unit_data.kappa(k) = solution(2);
    unit_data.gen_ko(k) = solution(7);
    unit_data.gen_delta(k) = solution(5);
    unit_data.gen_voltage(k) = solution(6);
    unit_data.gen_current(k) = solution(9);
    unit_data.omegaref = omegar;
    unit_data.power(k) = solution(8)*solution(3)*omegar;
    
    %qdimc = ((Ain/Aout)*r1*tan(beta1r))/(1 + cot((asin(nsolution(2)*sin(alpha1r))))*tan(beta1r));
    alpha1 = (asin(solution(2)*sin(alpha1r)));
    qdimc = solution(3)*((1 + cot(alpha1r)*tan(beta1r))/(1 + cot(alpha1)*tan(beta1r)));
    ms = ((1+psi)/mr)*ksi2*(abs(solution(1))/solution(2))*(cos(alpha1) + tan(alpha1r)*sin(alpha1));
    unit_data.etah(k) = (1-((Rf*solution(1)^2 + Ra*(solution(1)-qdimc)^2)/(H(end)/unit_data.Hr)));
    unit_data.eta(k) = (solution(1)*(ms - psi*solution(3))*unit_data.etah(k) - Rm*solution(3)^2)/(solution(1)*(H(end)/unit_data.Hr));
    
    

%% REJECTION 
elseif strcmp(scheme,'Rejection')
    
    unit_data.generatorstatus = 'offline';
    if strcmp(unit_data.turbinestatus,'open');  %IF TURBINE GUIDE VANES ARE OPEN
        %maxclosuretime = 5.2;
        maxclosuretime = 7.28;
        dkappa = dt/maxclosuretime;
        kappa = kappaprev - dkappa;
        
        alpha1 = (asin(kappa*sin(alpha1r)));
        ms = ((1+psi)/mr)*ksi2*(abs(qdim)/kappa)*(cos(alpha1) + tan(alpha1r)*sin(alpha1));
        EQ1 = Cp - B*Qr*qdim - unit_data.Hs - unit_data.Hr*(1/(1+sigmaloss))*((qdim/kappa)^2 + sigmaloss*(omegadim^2));
        EQ2 = qdim*(ms - psi*omegadim)*0.92 - Rm*omegadim^2 - (Ta/dt)*(omegadim - omegadimprev);
        
        nsolution = newton_n_dim(0.01,[qdimprev,omegadimprev],[qdim,omegadim],[EQ1;EQ2]);
        nsolution = double(nsolution);
        unit_data.flow(k) = nsolution(1)*Qr;
        unit_data.turb_omega(k) = nsolution(2)*omegar;
        unit_data.c(k) = kappa-kappaprev;
        unit_data.kappa(k) = kappa;
        unit_data.gen_ko(k) = 0;
        unit_data.gen_delta(k) = 0;
        unit_data.gen_voltage(k) = 0;
        unit_data.gen_current(k) = 0;
        unit_data.omegaref = omegar;
        
    elseif strcmp(unit_data.turbinestatus,'closed') %IF TURBINE GUIDE VANES ARE CLOSED
        EQ2 = - (Ta/dt)*(omegadim - omegadimprev) - Rm*omegadim^2;
        solution = solve(EQ2);
        solution= max(double(solution));
        
    	unit_data.flow(k) = 0.01;
        unit_data.turb_omega(k) = double(solution)*omegar;
        unit_data.c(k) = 0;
        unit_data.kappa(k) = 0.01;
        unit_data.gen_ko(k) = 0;
        unit_data.gen_delta(k) = 0;
        unit_data.gen_voltage(k) = 0;
        unit_data.gen_current(k) = 0;
        unit_data.omegaref = omegar;
        alpha1 = 0;
        
        
    end
    %IF unit is almost closed, set flow to small value to avoid division by
    %zero.
    if (unit_data.flow(k)/Qr) < 0.05
        unit_data.turbinestatus = 'closed';
    end
end
% DEBUG OUTPUT
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(['Unit nr: ',num2str(unit_data.unitID)])
disp(['Time is: ',num2str(t)])
disp(['Scheme is: ',scheme])
disp(['Kappa is: ',num2str(unit_data.kappa(k))])
disp(['C is: ',num2str(unit_data.kappa(k)-kappaprev)])
disp(['dC/dT is: ',num2str(unit_data.c(k)-cprev)])
disp(['Status is: ',unit_data.turbinestatus])
end