function [results] = DrivaSIM(datacarrier)
global unit1_data
global unit2_data
%% PRE PROCESSING
g = 9.81;                               %Gravity constant
a = datacarrier.wavespeed;              %Speed of sound in fluid
%---------------- SIMULATION DATA ------------------------
dt = datacarrier.dt;                    %Timestep
dx = dt*a;                              %Length increment
tmax = datacarrier.tmax;                %Simulation time
%---------------- DATA RESERVOIR -------------------------
Hres = datacarrier.Hr;                  %Reservoir reference head
Hout = 91;                              %Outlet reference head
Hr = Hres - Hout;                       %Turbine pressure without loss
%---------------- TUNNEL PROPERTIES ----------------------
f = datacarrier.frictionfactor;         %Friction factor tunnel
fpenstock = 0.0059*1.73;                %Friction factor penstock
%---------------- DATA HEADRACE1 --------------------------
A1 = 20;                                %Flow area tunnel
L1 = 15000;                             %Length tunnel
D1 = sqrt(4*A1/pi);                     %Diameter tunnel
N1 = L1/dx;                             %Number of sections
%----------------- DATA HEADRACE2 -------------------
A2 = 22;                                %Tunnel area
L2 = 5520; %+1dx due to valvesection OBS OBS OBS
D2 = sqrt(4*A2/pi);                     %Tunnel diameter
N2 = L2/dx;                             %Number of base tunnel elements
%------------------- DATA SURGE CHAMBER -------------------
surgeHeight = zeros(1,tmax/dt);         %Initiate dataholder surge chamber height
%------------------- DATA PENSTOCK -------------------
A3 = 3.46;                              %Tunnel area
L3 = 360;
D3 = sqrt(4*A3/pi);                     %Tunnel diameter
N3 = L3/dx;
%------------------- SETTING GRID POSITION INDICES -------------
pipestart1 = 1;                         %Start coordinate pipe 1
pipeend1 = N1;                          %End coordinate pipe 1
pipestart2 = pipeend1 + 1;              %Start coordinate pipe 2
pipeend2 = pipestart2 + N2;             %End coordinate pipe 2
surgepos = pipeend2 + 1;                %Coordinate of surge chamber
pipestart3 = surgepos + 1;              %Start coordinate penstock
pipeend3 = pipestart3 + N3;             %End coordinate penstock
valvepos = pipeend3 + 1;                %Coordinate of valve/turbine
%------------------- INPUT CHECK --------------------------
if dx/round(dx) ~= 1
    error('inconsistent CFL. Check a or dt')
end
if N1/round(N1) ~= 1
    error('length/section mismatch upstream. Check L1')
end
if N2/round(N2) ~= 1
    error(['length/section mismatch downstream ',num2str(N2/round(N2)),'. ','Check L2'])
end

R1 = f*dx/(2*g*(D1^5)*(pi/4)^2);        %headloss per section of pipe (rock)
R2 = f*dx/(2*g*(D2^5)*(pi/4)^2);        %headloss per section of pipe (rock)
R3 = fpenstock*dx/(2*g*(D3^5)*(pi/4)^2);%headloss per section of pipe (penstock)
B1 = a/(g*A1);                         	%MOC1 constant
B2 = a/(g*A2);                          %MOC2 constant
B3 = a/(g*A3);                          %MOC3 constant
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIMULATION INITIATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data = zeros(2,valvepos,tmax/dt);       %initiation of return data holder
Qp = zeros(1,valvepos);                 %Qp (flow for current time)
Hp = zeros(1,valvepos);                 %Hp (head for current time)
Q = zeros(1,valvepos);                  %Q (flow for previous time increment)
H = zeros(1,valvepos);                  %H (head for previous time increment)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DECLARING TURBINE UNITS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Declaring data unit 1
unit1_data.unitID = 1;                                  %Turbine nr for identification
unit1_data.turbinestatus = 'open';                      %Status of turbine(open/closed)
unit1_data.generatorstatus = 'online';                  %Status of generator(offline/online)
unit1_data.gen_delta = zeros(1,tmax/dt);                %Generator angular displacement  
unit1_data.gen_voltage = unit1_data.gen_delta;          %Generator output voltage
unit1_data.gen_current = unit1_data.gen_delta;          %Generator output current
unit1_data.gen_ko = unit1_data.gen_delta;               %Generator output current
unit1_data.turb_omega = unit1_data.gen_delta;           %Rotor angular speed
unit1_data.kappa = unit1_data.gen_delta;                %Guide vane opeing
unit1_data.c  = unit1_data.gen_delta;                   %Servo speed
unit1_data.flow = unit1_data.gen_delta;                 %Flow
unit1_data.power = unit1_data.gen_delta;                %Generator power w/o loss
unit1_data.etah = unit1_data.gen_delta;                 %Hydraulic efficiency
unit1_data.eta = unit1_data.gen_delta;                  %Total efficiency

% Collecting data and placing in dataholder for simulation routine
unit1_data.Rm = datacarrier.Rm1;
unit1_data.Ra = 0.6;
unit1_data.Rf = 0.0205;
unit1_data.deltat = datacarrier.deltat1;
unit1_data.deltab = datacarrier.deltab1;
unit1_data.Ti = datacarrier.Ti1;
unit1_data.Pref = datacarrier.Pref1;
unit1_data.Pnom = 65.37*10^6;
unit1_data.Hr = datacarrier.Hr1;
unit1_data.Qr = 13.12;
unit1_data.J = 118667;
unit1_data.deltaref = datacarrier.deltaref1;
unit1_data.Eref = datacarrier.Eref1;
unit1_data.deltatg = datacarrier.deltatg1;
unit1_data.deltabg = datacarrier.deltabg1;
unit1_data.genTi = datacarrier.genTi1;
unit1_data.simdata.t1 = datacarrier.t1_1;
unit1_data.simdata.t2 = datacarrier.t1_2;
unit1_data.simdata.t3 = datacarrier.t1_3;
unit1_data.simdata.mode1 = datacarrier.mode1_1;
unit1_data.simdata.mode2 = datacarrier.mode1_2;
unit1_data.simdata.mode3 = datacarrier.mode1_3;

%Declaring data unit 2
unit2_data.unitID = 2;
unit2_data.turbinestatus = 'open';
unit2_data.generatorstatus = 'online';
unit2_data.gen_delta = zeros(1,tmax/dt);                %Generator angular displacement  
unit2_data.gen_voltage = unit2_data.gen_delta;          %Generator output voltage
unit2_data.gen_current = unit2_data.gen_delta;          %Generator output current
unit2_data.gen_ko = unit2_data.gen_delta;               %Generator output current
unit2_data.turb_omega = unit2_data.gen_delta;           %Rotor angular speed
unit2_data.kappa = unit2_data.gen_delta;                %Guide vane opeing
unit2_data.c  = unit2_data.gen_delta;                   %Servo motor speed
unit2_data.flow = unit2_data.gen_delta;                 %Flow
unit2_data.power = unit2_data.gen_delta;
unit2_data.etah = unit2_data.gen_delta;                 %Hydraulic efficiency
unit2_data.eta = unit2_data.gen_delta;                  %Total efficiency

unit2_data.Rm = datacarrier.Rm2;
unit2_data.Ra = 0.6;
unit2_data.Rf = 0.01135;
unit2_data.deltat = datacarrier.deltat2;
unit2_data.deltab = datacarrier.deltab2;
unit2_data.Ti = datacarrier.Ti2;
unit2_data.Pref = datacarrier.Pref2;
unit2_data.Pnom = 69.5*10^6;
unit2_data.Hr = datacarrier.Hr2;
unit2_data.Qr = 13.81;
unit2_data.J = 82070;
unit2_data.deltaref = datacarrier.deltaref2;
unit2_data.Eref = datacarrier.Eref2;
unit2_data.deltatg = datacarrier.deltatg2;
unit2_data.deltabg = datacarrier.deltabg2;
unit2_data.genTi = datacarrier.genTi2;
unit2_data.simdata.t1 = datacarrier.t2_1;
unit2_data.simdata.t2 = datacarrier.t2_2;
unit2_data.simdata.t3 = datacarrier.t2_3;
unit2_data.simdata.mode1 = datacarrier.mode2_1;
unit2_data.simdata.mode2 = datacarrier.mode2_2;
unit2_data.simdata.mode3 = datacarrier.mode2_3;

% Finding initial flow and efficiency to yield required generator output
etagenerator = 0.982;                %Generator efficiency NOTE: MAY BE SET NUMERICALLY ELSEWHERE
P1 = 0;
P2 = 0;
dq = 0.001;
q1 = dq;
q2 = dq;
Ptarget1 = unit1_data.Pref*10^6/etagenerator;
Ptarget2 = unit2_data.Pref*10^6/etagenerator;
% NOTE: THE VARIABLES ABOVE SHOULD BE IMPORTED DIRECTLY VIA A FUNCTION. 
% THESE ARE SET BY FUNCTION AggregateMOCgenericMK2
alpha1r = 0.0615;
beta1r = 0.9562;
psi = 0.3020;
ksi2 = 1.305;
mr = 1.307;
Hs = -13.05;
hold on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x = 1;
while P1 <= Ptarget1 || P2 <= Ptarget2
    Q0 = q1*unit1_data.Qr + q2*unit2_data.Qr;
    for i = 1:pipeend1
        H(i) = Hr - (i-1)*R1*Q0*Q0;         %Head will drop due to resistance
        Q(i) = Q0;                          %Flow will remain the same
    end
    for i = pipestart2:pipeend2
        H(i) = H(i-1) - R2*Q0*Q0;         %Head will drop due to resistance
        Q(i) = Q0;                          %Flow will remain the same
    end
    H(surgepos) = H(surgepos-1);
    Q(surgepos) = Q(surgepos-1);
    for i = pipestart3:valvepos
        H(i) = H(i-1) - R3*Q0*Q0;         %Head will drop due to resistance
        Q(i) = Q0;                          %Flow will remain the same
    end

    % Finding startup variables for current iteration
    qdimc1 = ((1 + cot(alpha1r)*tan(beta1r))/(1 + cot(asin(q1*sin(alpha1r)))*tan(beta1r)));
    ms1 = ((1+psi)/mr)*ksi2*(cos(asin(q1*sin(alpha1r))) + tan(alpha1r)*sin(asin(q1*sin(alpha1r))));
    etah1 = (1-((unit1_data.Rf*q1^2 + unit1_data.Ra*(q1-qdimc1)^2)/(H(end)/unit1_data.Hr)));
    eta1 = (q1*(ms1 - psi)*etah1 - unit1_data.Rm)/(q1*(H(end)/unit1_data.Hr));
    
    qdimc2 = ((1 + cot(alpha1r)*tan(beta1r))/(1 + cot(asin(q2*sin(alpha1r)))*tan(beta1r)));
    ms2 = ((1+psi)/mr)*ksi2*(cos(asin(q2*sin(alpha1r))) + tan(alpha1r)*sin(asin(q2*sin(alpha1r))));
    etah2 = 1-((unit2_data.Rf*q2^2 + unit2_data.Ra*abs(q2-qdimc2)^2)/(H(end)/unit2_data.Hr));
    eta2 = (q2*(ms2 - psi)*etah2 - unit2_data.Rm)/(q2*(H(end)/unit2_data.Hr));
    
    P1 = eta1*9846.38*H(end)*q1*unit1_data.Qr;
    P2 = eta2*9846.38*H(end)*q2*unit2_data.Qr;
    
    %If target power is not reached, increase flow
    if P1 < Ptarget1
        q1 = q1 + dq;
    end
    if P2 < Ptarget2
        q2 = q2 + dq;
    end
    x=x+1;
    
end
% Find steady state for entire pipe, for computation of P-points
Q0 = q1*unit1_data.Qr + q2*unit2_data.Qr;
for i = 1:pipeend1
    H(i) = Hr - (i-1)*R1*Q0*Q0;         %Head will drop due to resistance
    Q(i) = Q0;                          %Flow will remain the same
end
for i = pipestart2:pipeend2
    H(i) = H(i-1) - R2*Q0*Q0;           %Head will drop due to resistance
    Q(i) = Q0;                          %Flow will remain the same
end
H(surgepos) = H(surgepos-1);            %Setting pressure in surgechamber (same node as the next)
Q(surgepos) = Q(surgepos-1);
for i = pipestart3:valvepos
    H(i) = H(i-1) - R3*Q0*Q0;           %Head will drop due to resistance
    Q(i) = Q0;                          %Flow will remain the same
end

data(1,:,1) = H;                        %Steady state properties (t = 0)
data(2,:,1) = Q;                        %"

t = 0;                                  %setting time
k = 1;                                  %counter

tic
%% SOLVER
while t < tmax
	%Boundary conditions RESERVOIR
    Hp(1) = Hr;
    Qp(1) = Q(2) + (Hp(1) - H(2) - R1*Q(2)*abs(Q(2)))/B1;
    
    %Computation of interior points PIPE 1
    for j = 2:pipeend1-1
       Cp = H(j-1) + Q(j-1)*(B1 - R1*abs(Q(j-1))); %MOC method
       Cm = H(j+1) - Q(j+1)*(B1 - R1*abs(Q(j+1)));
       Hp(j) = 0.5*(Cp+Cm);
       Qp(j) = (Cp-Hp(j))/B1;
    end
    %Boundary conditions junction pipe1/pipe2
    Cp = H(pipeend1-1) + Q(pipeend1-1)*(B1 - R1*abs(Q(pipeend1-1)));
    Cm = H(pipeend1+1) - Q(pipeend1+1)*(B2 - R2*abs(Q(pipeend1+1)));
    Qp(pipeend1) = (Cp - Cm)/(B1 + B2);
    Hp(pipeend1) = Cm + B2*Qp(pipeend1);

    %Computation of interior points PIPE 2
	for j = pipestart2:pipeend2-1
       Cp = H(j-1) + Q(j-1)*(B2 - R2*abs(Q(j-1)));
       Cm = H(j+1) - Q(j+1)*(B2 - R2*abs(Q(j+1)));
       Hp(j) = 0.5*(Cp+Cm);
       Qp(j) = (Cp-Hp(j))/B2;
    end
    %Boundary conditions junction pipe2/pipe3
    Cp = H(pipeend2-1) + Q(pipeend1-1)*(B2 - R2*abs(Q(pipeend1-1)));
    Cm = H(pipeend2+2) - Q(pipeend1+2)*(B3 - R3*abs(Q(pipeend1+2)));
    Qp(pipeend2) = (Cp - Cm)/(B2 + B3);
    Hp(pipeend2) = Cm + B3*Qp(pipeend2); 

    %Surge chamber
    [Hp,Qp,surgeHeight] = Surge_Chamber3(Q,H,Qp,Hp,surgepos,B2,B3,R2,R3,dt,surgeHeight,k);
    
    %Computation of interior points PIPE 3
    for j = pipestart3:pipeend3
       Cp = H(j-1) + Q(j-1)*(B3 - R3*abs(Q(j-1)));
       Cm = H(j+1) - Q(j+1)*(B3 - R3*abs(Q(j+1)));
       Hp(j) = 0.5*(Cp+Cm);
       Qp(j) = (Cp-Hp(j))/B3;
    end
    
    %Boundary conditions VALVE/Turbine
    [unit1_data] = AggregateMOCgenericMK2(t,H,B3,R3,unit1_data,k,dt,q1);
    [unit2_data] = AggregateMOCgenericMK2(t,H,B3,R3,unit2_data,k,dt,q2);
    %FOR ADDING MORE UNITS, DECLARE HERE AS WELL AS 1 MORE BLOCK OF
    %TURBINE DECLARATION ABOVE
    
    Cp = H(end-1) + Q(end-1)*(B3 - R3*abs(Q(end-1)));
    Qp(end) = unit1_data.flow(k) + unit2_data.flow(k);      %Flow in inlets are sum of both units
    Hp(end) = Cp - B3*Qp(end);                              %Calculating pressure
    
    %Data processing and preparation for next time iteration
	H = Hp;
	Q = Qp;
    data(1,:,k) = Hp;                   %Inserting fresh data into dataholder
    data(2,:,k) = Qp;                   %"
   
    t = t + dt;                         %Updating time counter
    k = k + 1;                          %Updating time vector position counter
   
    %Updating GUI
    percentage = floor(t/tmax*100);
    set(objectfinder('simstatus_percentage'),'string',[num2str(percentage),' %'])
    drawnow expose
end
%Creating resultsvector for return to GUI
results.H = data(1,:,:);
results.Q = data(2,:,:);
results.tmax = tmax;
results.dt = dt;
results.surgepos = surgepos;
results.turbinepos = valvepos;
results.surgeHeight = surgeHeight;
results.unit1_data = unit1_data;
results.unit2_data = unit2_data;
end