function [Hp,Qp,surgeheight] = Surge_Chamber(Q,H,surgepos,B,R,Vgas,z,dt,C,mode,n,Hchamber,k)

global C
global Vgas
global z
mode = 'enable';
%Chamber dimensions
SurgeD = 10;
SurgeH = 10;
SurgeL = 77.5;
%Chamber polytropic exponent
n = 1.2;
Vsurge = SurgeD*SurgeH*SurgeL;
Vgas0 = 4681;                           %Initial gas volume
Vfluid = Vsurge - Vgas0;                %Volume of fluid
z0 = Vfluid/(SurgeL*SurgeD);            %Starting water level

if isempty(C)
    Hjunction0 = H(surgepos);
    C = (Hjunction0 - z0)*(Vgas0)^n;    %Calculation of chamber constant
    Vgas = Vgas0;
    z = z0;
end

%Junction data
A3 = 12;
D3 = sqrt((4*A3)/pi);
f3 = 0.05;
L3 = 10;

if strcmp(mode,'enable')
    
    %From previous timestep
    Cp = H(surgepos-2) + Q(surgepos-2)*(B2 - R2*abs(Q(surgepos-2))); 
    Cm = H(surgepos+1) - Q(surgepos+1)*(B3 - R3*abs(Q(surgepos+1)));
    Q1 = Q(surgepos - 1);
    Q2 = Q(surgepos + 1);
    H4 = Hchamber - z;
    C2 = ((2*L3)/(g*A3*dt));
    C1 = H4 - H + ((f3*L3)/(g*D3*A3))*Q3*abs(Q3) - C2*Q3;
    
    %symbolic variable for the solver
    syms Hp
    syms Hp4
    syms Qp1
    syms Qp2
    syms Qp3
    syms znew

    %Equations to be solved
    EQ1 = C1 + C2*Qp3 + L3 - Hp + Hp4;
    EQ2 = (Hp4 + zint)*(Vgas - (0.5*dt*(Qp3 + Q3)))^n - C;
    EQ3 = Qp1 - Qp3 - Qp2;
    EQ4 = Cp - B*Qp1 - Hp;
    EQ5 = Cm + B*Qp1 - Hp;
    EQ6 = SurgeH - ((Vgas - 0.5*dt*(Qp3 + (Q1 - Q2)))/(SurgeD*SurgeL)) - zint;

    solutions = newton_n_dim(0.1,[H H4 Q1 Q2 Q3 z],[Hp Hp4 Qp1 Qp2 Qp3 zint],[EQ1;EQ2;EQ3;EQ4;EQ5;EQ6]);
    solutions = double(solutions);
    
    %Setting the new variables into dataholders
    Hp(surgepos) = solutions(1);
    Qp(surgepos) = nsolutions(4);
    chamberflow = nsolutions(5);

    Vgas = Vgas - 0.5*dt*(chamberflow + (Q1 - Q2));
    surgeheight(k) = SurgeH - (Vgas/(SurgeD*SurgeL));
    z = surgeheight(k);

elseif strcmp(mode,'disable') %For disabling the chamber
    Cp = H(surgepos-1) + Q(surgepos-1)*(B - R*abs(Q(surgepos-1)));
	Cm = H(surgepos+1) - Q(surgepos+1)*(B - R*abs(Q(surgepos+1))); 
	Hpipe = 0.5*(Cp+Cm);
	Qout = (Cp-Hpipe)/B;  
    Qin = NaN;
    Vgas = NaN;
    z = NaN;
end
end